@@ -4,6 +4,7 @@
  * Authors: Deepthi Devaki <deepthidevaki@multicorewareinc.com>,
  *          Rajesh Paulraj <rajesh@multicorewareinc.com>
  *          Praveen Kumar Tiwari <praveen@multicorewareinc.com>
+ *          Min Chen <chenm003@163.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,13 +28,15 @@
 #include "primitives.h"
 #include "x265.h"
 
-using namespace x265;
+using namespace X265_NS;
 
 #if _MSC_VER
 #pragma warning(disable: 4127) // conditional expression is constant, typical for templated functions
 #endif
 
 namespace {
+// file local namespace
+
 template<int width, int height>
 void filterPixelToShort_c(const pixel* src, intptr_t srcStride, int16_t* dst, intptr_t dstStride)
 {
@@ -53,7 +56,7 @@ void filterPixelToShort_c(const pixel* src, intptr_t srcStride, int16_t* dst, in
     }
 }
 
-void extendCURowColBorder(pixel* txt, intptr_t stride, int width, int height, int marginX)
+static void extendCURowColBorder(pixel* txt, intptr_t stride, int width, int height, int marginX)
 {
     for (int y = 0; y < height; y++)
     {
@@ -362,14 +365,14 @@ void filterVertical_sp_c(const int16_t* src, intptr_t srcStride, pixel* dst, int
 template<int N, int width, int height>
 void interp_hv_pp_c(const pixel* src, intptr_t srcStride, pixel* dst, intptr_t dstStride, int idxX, int idxY)
 {
-    short immedVals[(64 + 8) * (64 + 8)];
+    ALIGN_VAR_32(int16_t, immed[width * (height + N - 1)]);
 
-    interp_horiz_ps_c<N, width, height>(src, srcStride, immedVals, width, idxX, 1);
-    filterVertical_sp_c<N>(immedVals + 3 * width, width, dst, dstStride, width, height, idxY);
+    interp_horiz_ps_c<N, width, height>(src, srcStride, immed, width, idxX, 1);
+    filterVertical_sp_c<N>(immed + (N / 2 - 1) * width, width, dst, dstStride, width, height, idxY);
 }
 }
 
-namespace x265 {
+namespace X265_NS {
 // x265 private namespace
 
 #define CHROMA_420(W, H) \
