@@ -35,6 +35,109 @@ using namespace X265_NS;
 static uint64_t computeSSD(pixel *fenc, pixel *rec, intptr_t stride, uint32_t width, uint32_t height);
 static float calculateSSIM(pixel *pix1, intptr_t stride1, pixel *pix2, intptr_t stride2, uint32_t width, uint32_t height, void *buf, uint32_t& cnt);
 
+static void integral_init4h(uint32_t *sum, pixel *pix, intptr_t stride)
+{
+    int32_t v = pix[0] + pix[1] + pix[2] + pix[3];
+    for (int16_t x = 0; x < stride - 4; x++)
+    {
+        sum[x] = v + sum[x - stride];
+        v += pix[x + 4] - pix[x];
+    }
+}
+
+static void integral_init8h(uint32_t *sum, pixel *pix, intptr_t stride)
+{
+    int32_t v = pix[0] + pix[1] + pix[2] + pix[3] + pix[4] + pix[5] + pix[6] + pix[7];
+    for (int16_t x = 0; x < stride - 8; x++)
+    {
+        sum[x] = v + sum[x - stride];
+        v += pix[x + 8] - pix[x];
+    }
+}
+
+static void integral_init12h(uint32_t *sum, pixel *pix, intptr_t stride)
+{
+    int32_t v = pix[0] + pix[1] + pix[2] + pix[3] + pix[4] + pix[5] + pix[6] + pix[7] +
+        pix[8] + pix[9] + pix[10] + pix[11];
+    for (int16_t x = 0; x < stride - 12; x++)
+    {
+        sum[x] = v + sum[x - stride];
+        v += pix[x + 12] - pix[x];
+    }
+}
+
+static void integral_init16h(uint32_t *sum, pixel *pix, intptr_t stride)
+{
+    int32_t v = pix[0] + pix[1] + pix[2] + pix[3] + pix[4] + pix[5] + pix[6] + pix[7] +
+        pix[8] + pix[9] + pix[10] + pix[11] + pix[12] + pix[13] + pix[14] + pix[15];
+    for (int16_t x = 0; x < stride - 16; x++)
+    {
+        sum[x] = v + sum[x - stride];
+        v += pix[x + 16] - pix[x];
+    }
+}
+
+static void integral_init24h(uint32_t *sum, pixel *pix, intptr_t stride)
+{
+    int32_t v = pix[0] + pix[1] + pix[2] + pix[3] + pix[4] + pix[5] + pix[6] + pix[7] +
+        pix[8] + pix[9] + pix[10] + pix[11] + pix[12] + pix[13] + pix[14] + pix[15] +
+        pix[16] + pix[17] + pix[18] + pix[19] + pix[20] + pix[21] + pix[22] + pix[23];
+    for (int16_t x = 0; x < stride - 24; x++)
+    {
+        sum[x] = v + sum[x - stride];
+        v += pix[x + 24] - pix[x];
+    }
+}
+
+static void integral_init32h(uint32_t *sum, pixel *pix, intptr_t stride)
+{
+    int32_t v = pix[0] + pix[1] + pix[2] + pix[3] + pix[4] + pix[5] + pix[6] + pix[7] +
+        pix[8] + pix[9] + pix[10] + pix[11] + pix[12] + pix[13] + pix[14] + pix[15] +
+        pix[16] + pix[17] + pix[18] + pix[19] + pix[20] + pix[21] + pix[22] + pix[23] +
+        pix[24] + pix[25] + pix[26] + pix[27] + pix[28] + pix[29] + pix[30] + pix[31];
+    for (int16_t x = 0; x < stride - 32; x++)
+    {
+        sum[x] = v + sum[x - stride];
+        v += pix[x + 32] - pix[x];
+    }
+}
+
+static void integral_init4v(uint32_t *sum4, intptr_t stride)
+{
+    for (int x = 0; x < stride; x++)
+        sum4[x] = sum4[x + 4 * stride] - sum4[x];
+}
+
+static void integral_init8v(uint32_t *sum8, intptr_t stride)
+{
+    for (int x = 0; x < stride; x++)
+        sum8[x] = sum8[x + 8 * stride] - sum8[x];
+}
+
+static void integral_init12v(uint32_t *sum12, intptr_t stride)
+{
+    for (int x = 0; x < stride; x++)
+        sum12[x] = sum12[x + 12 * stride] - sum12[x];
+}
+
+static void integral_init16v(uint32_t *sum16, intptr_t stride)
+{
+    for (int x = 0; x < stride; x++)
+        sum16[x] = sum16[x + 16 * stride] - sum16[x];
+}
+
+static void integral_init24v(uint32_t *sum24, intptr_t stride)
+{
+    for (int x = 0; x < stride; x++)
+        sum24[x] = sum24[x + 24 * stride] - sum24[x];
+}
+
+static void integral_init32v(uint32_t *sum32, intptr_t stride)
+{
+    for (int x = 0; x < stride; x++)
+        sum32[x] = sum32[x + 32 * stride] - sum32[x];
+}
+
 void FrameFilter::destroy()
 {
     X265_FREE(m_ssimBuf);
@@ -65,6 +168,7 @@ void FrameFilter::init(Encoder *top, FrameEncoder *frame, int numRows, uint32_t
     m_saoRowDelay = m_param->bEnableLoopFilter ? 1 : 0;
     m_lastHeight = (m_param->sourceHeight % g_maxCUSize) ? (m_param->sourceHeight % g_maxCUSize) : g_maxCUSize;
     m_lastWidth = (m_param->sourceWidth % g_maxCUSize) ? (m_param->sourceWidth % g_maxCUSize) : g_maxCUSize;
+    integralCompleted.set(0);
 
     if (m_param->bEnableSsim)
         m_ssimBuf = X265_MALLOC(int, 8 * (m_param->sourceWidth / 4 + 3));
@@ -499,14 +603,19 @@ void FrameFilter::processRow(int row)
     if (!ctu->m_bFirstRowInSlice)
         processPostRow(row - 1);
 
-    if (ctu->m_bLastRowInSlice)
-        processPostRow(row);
-
     // NOTE: slices parallelism will be execute out-of-order
-    int numRowFinished;
-    for(numRowFinished = 0; numRowFinished < m_numRows; numRowFinished++)
-        if (!m_frame->m_reconRowFlag[numRowFinished].get())
-            break;
+    int numRowFinished = 0;
+    if (m_frame->m_reconRowFlag)
+    {
+        for (numRowFinished = 0; numRowFinished < m_numRows; numRowFinished++)
+        {
+            if (!m_frame->m_reconRowFlag[numRowFinished].get())
+                break;
+
+            if (numRowFinished == row)
+                continue;
+        }
+    }
 
     if (numRowFinished == m_numRows)
     {
@@ -522,6 +631,9 @@ void FrameFilter::processRow(int row)
             m_parallelFilter[0].m_sao.rdoSaoUnitRowEnd(saoParam, encData.m_slice->m_sps->numCUsInFrame);
         }
     }
+
+    if (ctu->m_bLastRowInSlice)
+        processPostRow(row);
 }
 
 void FrameFilter::processPostRow(int row)
@@ -656,6 +768,107 @@ void FrameFilter::processPostRow(int row)
         }
     } // end of (m_param->maxSlices == 1)
 
+    int lastRow = row == (int)m_frame->m_encData->m_slice->m_sps->numCuInHeight - 1;
+
+    /* generate integral planes for SEA motion search */
+    if (m_param->searchMethod == X265_SEA && m_frame->m_encData->m_meIntegral && m_frame->m_lowres.sliceType != X265_TYPE_B)
+    {
+        /* If WPP, other than first row, integral calculation for current row needs to wait till the
+        * integral for the previous row is computed */
+        if (m_param->bEnableWavefront && row)
+        {
+            while (m_parallelFilter[row - 1].m_frameFilter->integralCompleted.get() == 0)
+            {
+                m_parallelFilter[row - 1].m_frameFilter->integralCompleted.waitForChange(0);
+            }
+        }
+
+        int stride = (int)m_frame->m_reconPic->m_stride;
+        int padX = g_maxCUSize + 32;
+        int padY = g_maxCUSize + 16;
+        int numCuInHeight = m_frame->m_encData->m_slice->m_sps->numCuInHeight;
+        int maxHeight = numCuInHeight * g_maxCUSize;
+        int startRow = 0;
+
+        if (m_param->interlaceMode)
+            startRow = (row * g_maxCUSize >> 1);
+        else
+            startRow = row * g_maxCUSize;
+
+        int height = lastRow ? (maxHeight + g_maxCUSize * m_param->interlaceMode) : (((row + m_param->interlaceMode) * g_maxCUSize) + g_maxCUSize);
+
+        if (!row)
+        {
+            for (int i = 0; i < INTEGRAL_PLANE_NUM; i++)
+                memset(m_frame->m_encData->m_meIntegral[i] - padY * stride - padX, 0, stride * sizeof(uint32_t));
+            startRow = -padY;
+        }
+
+        if (lastRow)
+            height += padY - 1;
+
+        for (int y = startRow; y < height; y++)
+        {
+            pixel    *pix = m_frame->m_reconPic->m_picOrg[0] + y * stride - padX;
+            uint32_t *sum32x32 = m_frame->m_encData->m_meIntegral[0] + (y + 1) * stride - padX;
+            uint32_t *sum32x24 = m_frame->m_encData->m_meIntegral[1] + (y + 1) * stride - padX;
+            uint32_t *sum32x8 = m_frame->m_encData->m_meIntegral[2] + (y + 1) * stride - padX;
+            uint32_t *sum24x32 = m_frame->m_encData->m_meIntegral[3] + (y + 1) * stride - padX;
+            uint32_t *sum16x16 = m_frame->m_encData->m_meIntegral[4] + (y + 1) * stride - padX;
+            uint32_t *sum16x12 = m_frame->m_encData->m_meIntegral[5] + (y + 1) * stride - padX;
+            uint32_t *sum16x4 = m_frame->m_encData->m_meIntegral[6] + (y + 1) * stride - padX;
+            uint32_t *sum12x16 = m_frame->m_encData->m_meIntegral[7] + (y + 1) * stride - padX;
+            uint32_t *sum8x32 = m_frame->m_encData->m_meIntegral[8] + (y + 1) * stride - padX;
+            uint32_t *sum8x8 = m_frame->m_encData->m_meIntegral[9] + (y + 1) * stride - padX;
+            uint32_t *sum4x16 = m_frame->m_encData->m_meIntegral[10] + (y + 1) * stride - padX;
+            uint32_t *sum4x4 = m_frame->m_encData->m_meIntegral[11] + (y + 1) * stride - padX;
+
+            /*For width = 32 */
+            integral_init32h(sum32x32, pix, stride);
+            if (y >= 32 - padY)
+                integral_init32v(sum32x32 - 32 * stride, stride);
+            integral_init32h(sum32x24, pix, stride);
+            if (y >= 24 - padY)
+                integral_init24v(sum32x24 - 24 * stride, stride);
+            integral_init32h(sum32x8, pix, stride);
+            if (y >= 8 - padY)
+                integral_init8v(sum32x8 - 8 * stride, stride);
+            /*For width = 24 */
+            integral_init24h(sum24x32, pix, stride);
+            if (y >= 32 - padY)
+                integral_init32v(sum24x32 - 32 * stride, stride);
+            /*For width = 16 */
+            integral_init16h(sum16x16, pix, stride);
+            if (y >= 16 - padY)
+                integral_init16v(sum16x16 - 16 * stride, stride);
+            integral_init16h(sum16x12, pix, stride);
+            if (y >= 12 - padY)
+                integral_init12v(sum16x12 - 12 * stride, stride);
+            integral_init16h(sum16x4, pix, stride);
+            if (y >= 4 - padY)
+                integral_init4v(sum16x4 - 4 * stride, stride);
+            /*For width = 12 */
+            integral_init12h(sum12x16, pix, stride);
+            if (y >= 16 - padY)
+                integral_init16v(sum12x16 - 16 * stride, stride);
+            /*For width = 8 */
+            integral_init8h(sum8x32, pix, stride);
+            if (y >= 32 - padY)
+                integral_init32v(sum8x32 - 32 * stride, stride);
+            integral_init8h(sum8x8, pix, stride);
+            if (y >= 8 - padY)
+                integral_init8v(sum8x8 - 8 * stride, stride);
+            /*For width = 4 */
+            integral_init4h(sum4x16, pix, stride);
+            if (y >= 16 - padY)
+                integral_init16v(sum4x16 - 16 * stride, stride);
+            integral_init4h(sum4x4, pix, stride);
+            if (y >= 4 - padY)
+                integral_init4v(sum4x4 - 4 * stride, stride);
+        }
+        m_parallelFilter[row].m_frameFilter->integralCompleted.set(1);
+    }
+
     if (ATOMIC_INC(&m_frameEncoder->m_completionCount) == 2 * (int)m_frameEncoder->m_numRows)
     {
         m_frameEncoder->m_completionEvent.trigger();
