@@ -109,6 +109,8 @@ MotionEstimate::MotionEstimate()
     blockOffset = 0;
     bChromaSATD = false;
     chromaSatd = NULL;
+    for (int i = 0; i < INTEGRAL_PLANE_NUM; i++)
+        integral[i] = NULL;
 }
 
 void MotionEstimate::init(int csp)
@@ -165,10 +167,12 @@ void MotionEstimate::setSourcePU(pixel *fencY, intptr_t stride, intptr_t offset,
     partEnum = partitionFromSizes(pwidth, pheight);
     X265_CHECK(LUMA_4x4 != partEnum, "4x4 inter partition detected!\n");
     sad = primitives.pu[partEnum].sad;
+    ads = primitives.pu[partEnum].ads;
     satd = primitives.pu[partEnum].satd;
     sad_x3 = primitives.pu[partEnum].sad_x3;
     sad_x4 = primitives.pu[partEnum].sad_x4;
 
+
     blockwidth = pwidth;
     blockOffset = offset;
     absPartIdx = ctuAddr = -1;
@@ -188,6 +192,7 @@ void MotionEstimate::setSourcePU(const Yuv& srcFencYuv, int _ctuAddr, int cuPart
     partEnum = partitionFromSizes(pwidth, pheight);
     X265_CHECK(LUMA_4x4 != partEnum, "4x4 inter partition detected!\n");
     sad = primitives.pu[partEnum].sad;
+    ads = primitives.pu[partEnum].ads;
     satd = primitives.pu[partEnum].satd;
     sad_x3 = primitives.pu[partEnum].sad_x3;
     sad_x4 = primitives.pu[partEnum].sad_x4;
@@ -278,12 +283,31 @@ void MotionEstimate::setSourcePU(const Yuv& srcFencYuv, int _ctuAddr, int cuPart
         costs[1] += mvcost((omv + MV(m1x, m1y)) << 2); \
         costs[2] += mvcost((omv + MV(m2x, m2y)) << 2); \
         costs[3] += mvcost((omv + MV(m3x, m3y)) << 2); \
-        COPY2_IF_LT(bcost, costs[0], bmv, omv + MV(m0x, m0y)); \
-        COPY2_IF_LT(bcost, costs[1], bmv, omv + MV(m1x, m1y)); \
-        COPY2_IF_LT(bcost, costs[2], bmv, omv + MV(m2x, m2y)); \
-        COPY2_IF_LT(bcost, costs[3], bmv, omv + MV(m3x, m3y)); \
+        if ((omv.y + m0y >= mvmin.y) & (omv.y + m0y <= mvmax.y)) \
+            COPY2_IF_LT(bcost, costs[0], bmv, omv + MV(m0x, m0y)); \
+        if ((omv.y + m1y >= mvmin.y) & (omv.y + m1y <= mvmax.y)) \
+            COPY2_IF_LT(bcost, costs[1], bmv, omv + MV(m1x, m1y)); \
+        if ((omv.y + m2y >= mvmin.y) & (omv.y + m2y <= mvmax.y)) \
+            COPY2_IF_LT(bcost, costs[2], bmv, omv + MV(m2x, m2y)); \
+        if ((omv.y + m3y >= mvmin.y) & (omv.y + m3y <= mvmax.y)) \
+            COPY2_IF_LT(bcost, costs[3], bmv, omv + MV(m3x, m3y)); \
     }
 
+#define COST_MV_X3_ABS( m0x, m0y, m1x, m1y, m2x, m2y )\
+{\
+    sad_x3(fenc, \
+    fref + (m0x) + (m0y) * stride, \
+    fref + (m1x) + (m1y) * stride, \
+    fref + (m2x) + (m2y) * stride, \
+    stride, costs); \
+    costs[0] += p_cost_mvx[(m0x) << 2]; /* no cost_mvy */\
+    costs[1] += p_cost_mvx[(m1x) << 2]; \
+    costs[2] += p_cost_mvx[(m2x) << 2]; \
+    COPY3_IF_LT(bcost, costs[0], bmv.x, m0x, bmv.y, m0y); \
+    COPY3_IF_LT(bcost, costs[1], bmv.x, m1x, bmv.y, m1y); \
+    COPY3_IF_LT(bcost, costs[2], bmv.x, m2x, bmv.y, m2y); \
+}
+
 #define COST_MV_X4_DIR(m0x, m0y, m1x, m1y, m2x, m2y, m3x, m3y, costs) \
     { \
         pixel *pix_base = fref + bmv.x + bmv.y * stride; \
@@ -627,6 +651,7 @@ int MotionEstimate::motionEstimate(ReferencePlanes *ref,
         {
             bcost = cost;
             bmv = 0;
+            bmv.y = X265_MAX(X265_MIN(0, mvmax.y), mvmin.y);
         }
     }
 
@@ -659,8 +684,10 @@ int MotionEstimate::motionEstimate(ReferencePlanes *ref,
         do
         {
             COST_MV_X4_DIR(0, -1, 0, 1, -1, 0, 1, 0, costs);
-            COPY1_IF_LT(bcost, (costs[0] << 4) + 1);
-            COPY1_IF_LT(bcost, (costs[1] << 4) + 3);
+            if ((bmv.y - 1 >= mvmin.y) & (bmv.y - 1 <= mvmax.y))
+                COPY1_IF_LT(bcost, (costs[0] << 4) + 1);
+            if ((bmv.y + 1 >= mvmin.y) & (bmv.y + 1 <= mvmax.y))
+                COPY1_IF_LT(bcost, (costs[1] << 4) + 3);
             COPY1_IF_LT(bcost, (costs[2] << 4) + 4);
             COPY1_IF_LT(bcost, (costs[3] << 4) + 12);
             if (!(bcost & 15))
@@ -698,36 +725,57 @@ me_hex2:
       /* equivalent to the above, but eliminates duplicate candidates */
         COST_MV_X3_DIR(-2, 0, -1, 2,  1, 2, costs);
         bcost <<= 3;
-        COPY1_IF_LT(bcost, (costs[0] << 3) + 2);
-        COPY1_IF_LT(bcost, (costs[1] << 3) + 3);
-        COPY1_IF_LT(bcost, (costs[2] << 3) + 4);
+        if ((bmv.y >= mvmin.y) & (bmv.y <= mvmax.y))
+            COPY1_IF_LT(bcost, (costs[0] << 3) + 2);
+        if ((bmv.y + 2 >= mvmin.y) & (bmv.y + 2 <= mvmax.y))
+        {
+            COPY1_IF_LT(bcost, (costs[1] << 3) + 3);
+            COPY1_IF_LT(bcost, (costs[2] << 3) + 4);
+        }
+
         COST_MV_X3_DIR(2, 0,  1, -2, -1, -2, costs);
-        COPY1_IF_LT(bcost, (costs[0] << 3) + 5);
-        COPY1_IF_LT(bcost, (costs[1] << 3) + 6);
-        COPY1_IF_LT(bcost, (costs[2] << 3) + 7);
+        if ((bmv.y >= mvmin.y) & (bmv.y <= mvmax.y))
+            COPY1_IF_LT(bcost, (costs[0] << 3) + 5);
+        if ((bmv.y - 2 >= mvmin.y) & (bmv.y - 2 <= mvmax.y))
+        {
+            COPY1_IF_LT(bcost, (costs[1] << 3) + 6);
+            COPY1_IF_LT(bcost, (costs[2] << 3) + 7);
+        }
 
         if (bcost & 7)
         {
             int dir = (bcost & 7) - 2;
-            bmv += hex2[dir + 1];
 
-            /* half hexagon, not overlapping the previous iteration */
-            for (int i = (merange >> 1) - 1; i > 0 && bmv.checkRange(mvmin, mvmax); i--)
+            if ((bmv.y + hex2[dir + 1].y >= mvmin.y) & (bmv.y + hex2[dir + 1].y <= mvmax.y))
             {
-                COST_MV_X3_DIR(hex2[dir + 0].x, hex2[dir + 0].y,
-                               hex2[dir + 1].x, hex2[dir + 1].y,
-                               hex2[dir + 2].x, hex2[dir + 2].y,
-                               costs);
-                bcost &= ~7;
-                COPY1_IF_LT(bcost, (costs[0] << 3) + 1);
-                COPY1_IF_LT(bcost, (costs[1] << 3) + 2);
-                COPY1_IF_LT(bcost, (costs[2] << 3) + 3);
-                if (!(bcost & 7))
-                    break;
-                dir += (bcost & 7) - 2;
-                dir = mod6m1[dir + 1];
                 bmv += hex2[dir + 1];
-            }
+
+                /* half hexagon, not overlapping the previous iteration */
+                for (int i = (merange >> 1) - 1; i > 0 && bmv.checkRange(mvmin, mvmax); i--)
+                {
+                    COST_MV_X3_DIR(hex2[dir + 0].x, hex2[dir + 0].y,
+                        hex2[dir + 1].x, hex2[dir + 1].y,
+                        hex2[dir + 2].x, hex2[dir + 2].y,
+                        costs);
+                    bcost &= ~7;
+
+                    if ((bmv.y + hex2[dir + 0].y >= mvmin.y) & (bmv.y + hex2[dir + 0].y <= mvmax.y))
+                        COPY1_IF_LT(bcost, (costs[0] << 3) + 1);
+
+                    if ((bmv.y + hex2[dir + 1].y >= mvmin.y) & (bmv.y + hex2[dir + 1].y <= mvmax.y))
+                        COPY1_IF_LT(bcost, (costs[1] << 3) + 2);
+
+                    if ((bmv.y + hex2[dir + 2].y >= mvmin.y) & (bmv.y + hex2[dir + 2].y <= mvmax.y))
+                        COPY1_IF_LT(bcost, (costs[2] << 3) + 3);
+
+                    if (!(bcost & 7))
+                        break;
+
+                    dir += (bcost & 7) - 2;
+                    dir = mod6m1[dir + 1];
+                    bmv += hex2[dir + 1];
+                }
+            } // if ((bmv.y + hex2[dir + 1].y >= mvmin.y) & (bmv.y + hex2[dir + 1].y <= mvmax.y))
         }
         bcost >>= 3;
 #endif // if 0
@@ -735,15 +783,21 @@ me_hex2:
         /* square refine */
         int dir = 0;
         COST_MV_X4_DIR(0, -1,  0, 1, -1, 0, 1, 0, costs);
-        COPY2_IF_LT(bcost, costs[0], dir, 1);
-        COPY2_IF_LT(bcost, costs[1], dir, 2);
+        if ((bmv.y - 1 >= mvmin.y) & (bmv.y - 1 <= mvmax.y))
+            COPY2_IF_LT(bcost, costs[0], dir, 1);
+        if ((bmv.y + 1 >= mvmin.y) & (bmv.y + 1 <= mvmax.y))
+            COPY2_IF_LT(bcost, costs[1], dir, 2);
         COPY2_IF_LT(bcost, costs[2], dir, 3);
         COPY2_IF_LT(bcost, costs[3], dir, 4);
         COST_MV_X4_DIR(-1, -1, -1, 1, 1, -1, 1, 1, costs);
-        COPY2_IF_LT(bcost, costs[0], dir, 5);
-        COPY2_IF_LT(bcost, costs[1], dir, 6);
-        COPY2_IF_LT(bcost, costs[2], dir, 7);
-        COPY2_IF_LT(bcost, costs[3], dir, 8);
+        if ((bmv.y - 1 >= mvmin.y) & (bmv.y - 1 <= mvmax.y))
+            COPY2_IF_LT(bcost, costs[0], dir, 5);
+        if ((bmv.y + 1 >= mvmin.y) & (bmv.y + 1 <= mvmax.y))
+            COPY2_IF_LT(bcost, costs[1], dir, 6);
+        if ((bmv.y - 1 >= mvmin.y) & (bmv.y - 1 <= mvmax.y))
+            COPY2_IF_LT(bcost, costs[2], dir, 7);
+        if ((bmv.y + 1 >= mvmin.y) & (bmv.y + 1 <= mvmax.y))
+            COPY2_IF_LT(bcost, costs[3], dir, 8);
         bmv += square1[dir];
         break;
     }
@@ -756,6 +810,7 @@ me_hex2:
         /* refine predictors */
         omv = bmv;
         ucost1 = bcost;
+        X265_CHECK(((pmv.y >= mvmin.y) & (pmv.y <= mvmax.y)), "pmv outside of search range!");
         DIA1_ITER(pmv.x, pmv.y);
         if (pmv.notZero())
             DIA1_ITER(0, 0);
@@ -879,7 +934,7 @@ me_hex2:
            stride, costs + 4 * k); \
     fref_base += 2 * dy;
 #define ADD_MVCOST(k, x, y) costs[k] += p_cost_omvx[x * 4 * i] + p_cost_omvy[y * 4 * i]
-#define MIN_MV(k, x, y)     COPY2_IF_LT(bcost, costs[k], dir, x * 16 + (y & 15))
+#define MIN_MV(k, dx, dy)     if ((omv.y + (dy) >= mvmin.y) & (omv.y + (dy) <= mvmax.y)) { COPY2_IF_LT(bcost, costs[k], dir, dx * 16 + (dy & 15)) }
 
                 SADS(0, +0, -4, +0, +4, -2, -3, +2, -3);
                 SADS(1, -4, -2, +4, -2, -4, -1, +4, -1);
@@ -1043,6 +1098,161 @@ me_hex2:
         break;
     }
 
+    case X265_SEA:
+    {
+        // Successive Elimination Algorithm
+        const int16_t minX = X265_MAX(omv.x - (int16_t)merange, mvmin.x);
+        const int16_t minY = X265_MAX(omv.y - (int16_t)merange, mvmin.y);
+        const int16_t maxX = X265_MIN(omv.x + (int16_t)merange, mvmax.x);
+        const int16_t maxY = X265_MIN(omv.y + (int16_t)merange, mvmax.y);
+        const uint16_t *p_cost_mvx = m_cost_mvx - qmvp.x;
+        const uint16_t *p_cost_mvy = m_cost_mvy - qmvp.y;
+        int16_t* meScratchBuffer = NULL;
+        int scratchSize = merange * 2 + 4;
+        if (scratchSize)
+        {
+            meScratchBuffer = X265_MALLOC(int16_t, scratchSize);
+            memset(meScratchBuffer, 0, sizeof(int16_t)* scratchSize);
+        }
+
+        /* SEA is fastest in multiples of 4 */
+        int meRangeWidth = (maxX - minX + 3) & ~3;
+        int w = 0, h = 0;                    // Width and height of the PU
+        ALIGN_VAR_32(pixel, zero[64 * FENC_STRIDE]) = { 0 };
+        ALIGN_VAR_32(int, encDC[4]);
+        uint16_t *fpelCostMvX = m_fpelMvCosts[-qmvp.x & 3] + (-qmvp.x >> 2);
+        sizesFromPartition(partEnum, &w, &h);
+        int deltaX = (w <= 8) ? (w) : (w >> 1);
+        int deltaY = (h <= 8) ? (h) : (h >> 1);
+
+        /* Check if very small rectangular blocks which cannot be sub-divided anymore */
+        bool smallRectPartition = partEnum == LUMA_4x4 || partEnum == LUMA_16x12 ||
+            partEnum == LUMA_12x16 || partEnum == LUMA_16x4 || partEnum == LUMA_4x16;
+        /* Check if vertical partition */
+        bool verticalRect = partEnum == LUMA_32x64 || partEnum == LUMA_16x32 || partEnum == LUMA_8x16 ||
+            partEnum == LUMA_4x8;
+        /* Check if horizontal partition */
+        bool horizontalRect = partEnum == LUMA_64x32 || partEnum == LUMA_32x16 || partEnum == LUMA_16x8 ||
+            partEnum == LUMA_8x4;
+        /* Check if assymetric vertical partition */
+        bool assymetricVertical = partEnum == LUMA_12x16 || partEnum == LUMA_4x16 || partEnum == LUMA_24x32 ||
+            partEnum == LUMA_8x32 || partEnum == LUMA_48x64 || partEnum == LUMA_16x64;
+        /* Check if assymetric horizontal partition */
+        bool assymetricHorizontal = partEnum == LUMA_16x12 || partEnum == LUMA_16x4 || partEnum == LUMA_32x24 ||
+            partEnum == LUMA_32x8 || partEnum == LUMA_64x48 || partEnum == LUMA_64x16;
+
+        int tempPartEnum = 0;
+
+        /* If a vertical rectangular partition, it is horizontally split into two, for ads_x2() */
+        if (verticalRect)
+            tempPartEnum = partitionFromSizes(w, h >> 1);
+        /* If a horizontal rectangular partition, it is vertically split into two, for ads_x2() */
+        else if (horizontalRect)
+            tempPartEnum = partitionFromSizes(w >> 1, h);
+        /* We have integral planes introduced to account for assymetric partitions.
+         * Hence all assymetric partitions except those which cannot be split into legal sizes,
+         * are split into four for ads_x4() */
+        else if (assymetricVertical || assymetricHorizontal)
+            tempPartEnum = smallRectPartition ? partEnum : partitionFromSizes(w >> 1, h >> 1);
+        /* General case: Square partitions. All partitions with width > 8 are split into four
+         * for ads_x4(), for 4x4 and 8x8 we do ads_x1() */
+        else
+            tempPartEnum = (w <= 8) ? partEnum : partitionFromSizes(w >> 1, h >> 1);
+
+        /* Successive elimination by comparing DC before a full SAD,
+         * because sum(abs(diff)) >= abs(diff(sum)). */
+        primitives.pu[tempPartEnum].sad_x4(zero,
+                         fenc,
+                         fenc + deltaX,
+                         fenc + deltaY * FENC_STRIDE,
+                         fenc + deltaX + deltaY * FENC_STRIDE,
+                         FENC_STRIDE,
+                         encDC);
+
+        /* Assigning appropriate integral plane */
+        uint32_t *sumsBase = NULL;
+        switch (deltaX)
+        {
+            case 32: if (deltaY % 24 == 0)
+                         sumsBase = integral[1];
+                     else if (deltaY == 8)
+                         sumsBase = integral[2];
+                     else
+                         sumsBase = integral[0];
+               break;
+            case 24: sumsBase = integral[3];
+               break;
+            case 16: if (deltaY % 12 == 0)
+                         sumsBase = integral[5];
+                     else if (deltaY == 4)
+                         sumsBase = integral[6];
+                     else
+                         sumsBase = integral[4];
+               break;
+            case 12: sumsBase = integral[7];
+                break;
+            case 8: if (deltaY == 32)
+                        sumsBase = integral[8];
+                    else
+                        sumsBase = integral[9];
+                break;
+            case 4: if (deltaY == 16)
+                        sumsBase = integral[10];
+                    else
+                        sumsBase = integral[11];
+                break;
+            default: sumsBase = integral[11];
+                break;
+        }
+
+        if (partEnum == LUMA_64x64 || partEnum == LUMA_32x32 || partEnum == LUMA_16x16 ||
+            partEnum == LUMA_32x64 || partEnum == LUMA_16x32 || partEnum == LUMA_8x16 ||
+            partEnum == LUMA_4x8 || partEnum == LUMA_12x16 || partEnum == LUMA_4x16 ||
+            partEnum == LUMA_24x32 || partEnum == LUMA_8x32 || partEnum == LUMA_48x64 ||
+            partEnum == LUMA_16x64)
+            deltaY *= (int)stride;
+
+        if (verticalRect)
+            encDC[1] = encDC[2];
+
+        if (horizontalRect)
+            deltaY = deltaX;
+
+        /* ADS and SAD */
+        MV tmv;
+        for (tmv.y = minY; tmv.y <= maxY; tmv.y++)
+        {
+            int i, xn;
+            int ycost = p_cost_mvy[tmv.y] << 2;
+            if (bcost <= ycost)
+                continue;
+            bcost -= ycost;
+
+            /* ADS_4 for 16x16, 32x32, 64x64, 24x32, 32x24, 48x64, 64x48, 32x8, 8x32, 64x16, 16x64 partitions
+             * ADS_1 for 4x4, 8x8, 16x4, 4x16, 16x12, 12x16 partitions
+             * ADS_2 for all other rectangular partitions */
+            xn = ads(encDC,
+                    sumsBase + minX + tmv.y * stride,
+                    deltaY,
+                    fpelCostMvX + minX,
+                    meScratchBuffer,
+                    meRangeWidth,
+                    bcost);
+
+            for (i = 0; i < xn - 2; i += 3)
+                COST_MV_X3_ABS(minX + meScratchBuffer[i], tmv.y,
+                             minX + meScratchBuffer[i + 1], tmv.y,
+                             minX + meScratchBuffer[i + 2], tmv.y);
+
+            bcost += ycost;
+            for (; i < xn; i++)
+                COST_MV(minX + meScratchBuffer[i], tmv.y);
+        }
+        if (meScratchBuffer)
+            x265_free(meScratchBuffer);
+        break;
+    }
+
     case X265_FULL_SEARCH:
     {
         // dead slow exhaustive search, but at least it uses sad_x4()
@@ -1099,6 +1309,7 @@ me_hex2:
     if ((g_maxSlices > 1) & ((bmv.y < qmvmin.y) | (bmv.y > qmvmax.y)))
     {
         bmv.y = x265_min(x265_max(bmv.y, qmvmin.y), qmvmax.y);
+        bcost = subpelCompare(ref, bmv, satd) + mvcost(bmv);
     }
 
     if (!bcost)
@@ -1113,6 +1324,11 @@ me_hex2:
         for (int i = 1; i <= wl.hpel_dirs; i++)
         {
             MV qmv = bmv + square1[i] * 2;
+
+            /* skip invalid range */
+            if ((qmv.y < qmvmin.y) | (qmv.y > qmvmax.y))
+                continue;
+
             int cost = ref->lowresQPelCost(fenc, blockOffset, qmv, sad) + mvcost(qmv);
             COPY2_IF_LT(bcost, cost, bdir, i);
         }
@@ -1124,6 +1340,11 @@ me_hex2:
         for (int i = 1; i <= wl.qpel_dirs; i++)
         {
             MV qmv = bmv + square1[i];
+
+            /* skip invalid range */
+            if ((qmv.y < qmvmin.y) | (qmv.y > qmvmax.y))
+                continue;
+
             int cost = ref->lowresQPelCost(fenc, blockOffset, qmv, satd) + mvcost(qmv);
             COPY2_IF_LT(bcost, cost, bdir, i);
         }
@@ -1150,7 +1371,7 @@ me_hex2:
                 MV qmv = bmv + square1[i] * 2;
 
                 // check mv range for slice bound
-                if ((g_maxSlices > 1) & ((qmv.y < qmvmin.y) | (qmv.y > qmvmax.y)))
+                if ((qmv.y < qmvmin.y) | (qmv.y > qmvmax.y))
                     continue;
 
                 int cost = subpelCompare(ref, qmv, hpelcomp) + mvcost(qmv);
@@ -1175,7 +1396,7 @@ me_hex2:
                 MV qmv = bmv + square1[i];
 
                 // check mv range for slice bound
-                if ((g_maxSlices > 1) & ((qmv.y < qmvmin.y) | (qmv.y > qmvmax.y)))
+                if ((qmv.y < qmvmin.y) | (qmv.y > qmvmax.y))
                     continue;
 
                 int cost = subpelCompare(ref, qmv, satd) + mvcost(qmv);
@@ -1189,6 +1410,9 @@ me_hex2:
         }
     }
 
+    // check mv range for slice bound
+    X265_CHECK(((bmv.y >= qmvmin.y) & (bmv.y <= qmvmax.y)), "mv beyond range!");
+
     x265_emms();
     outQMv = bmv;
     return bcost;
